<?php

/**
 * @file
 * Implement an gridstack field.
 */

/**
 * Implements hook_field_info().
 */
function gridstack_field_field_info() {
  return array(
    'gridstack_field' => array(
      'label' => t('Gridstack field'),
      'description' => t('Implements gridtack plugin'),
      'default_widget' => 'gridstack_field_widget',
      'default_formatter' => 'gridstack_field_formatter',
      'settings' => array(
        'height' => '0',
        'width' => '12',
        'cellHeight' => '60',
        'minWidth' => '768',
        'rtl' => 'auto',
        'verticalMargin' => '10',
        'animate' => 0,
        'alwaysShowResizeHandle' => 0,
        'auto' => 1,
        'disableDrag' => 0,
        'disableResize' => 0,
        'float' => 0,
      ),
    ),
  );
}

/**
 * Implements hook_field_widget_info().
 */
function gridstack_field_field_widget_info() {
  return array(
    'gridstack_field' => array(
      'label' => t('Gridstack field'),
      'field types' => array('gridstack_field'),
    ),
  );
}

/**
 * Implements hook_field_settings_form().
 */
function gridstack_field_field_settings_form($field, $instance) {
  $defaults['row_setting'] = field_info_field_settings($field['type']);
  $settings = array_merge($defaults, $field['settings']);

  foreach (node_type_get_types() as $key => $type) {
    $form[$key] = array(
      '#type'  => 'checkbox',
      '#title' => check_plain($type->name),
      '#default_value' => !empty($settings[$key]) ? $settings[$key] : 0,
    );
    $displays = _gridstack_field_get_displays($key);
    $form[$key . '_display'] = array(
      '#type'   => 'container',
      '#states' => array(
        'visible' => array(
          ':input[name="field[settings][' . $key . ']"]' => array('checked' => TRUE),
        ),
      ),
    );
    $form[$key . '_display']['settings'] = array(
      '#type'    => 'radios',
      '#options' => $displays,
      '#default_value' => !empty($settings[$key . '_display']) ? $settings[$key . '_display']['settings'] : 0,
    );
  }

  // Settings for Gridstack plugin.
  $form['row_setting'] = array(
    '#type' => 'fieldset',
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
    '#title' => t('Row settings'),
  );
  $form['row_setting']['height'] = array(
    '#title' => t('Height'),
    '#type' => 'textfield',
    '#default_value' => $settings['row_setting']['height'],
    '#maxlength' => 60,
    '#size' => 60,
  );
  $form['row_setting']['width'] = array(
    '#title' => t('Width'),
    '#type' => 'textfield',
    '#default_value' => $settings['row_setting']['width'],
    '#maxlength' => 60,
    '#size' => 60,
  );
  $form['row_setting']['cellHeight'] = array(
    '#title' => t('Cell height'),
    '#type' => 'textfield',
    '#default_value' => $settings['row_setting']['cellHeight'],
    '#maxlength' => 60,
    '#size' => 60,
  );
  $form['row_setting']['minWidth'] = array(
    '#title' => t('Min width'),
    '#type' => 'textfield',
    '#default_value' => $settings['row_setting']['minWidth'],
    '#maxlength' => 60,
    '#size' => 60,
  );
  $form['row_setting']['rtl'] = array(
    '#title' => t('RTL'),
    '#type' => 'textfield',
    '#default_value' => $settings['row_setting']['rtl'],
    '#maxlength' => 60,
    '#size' => 60,
  );
  $form['row_setting']['verticalMargin'] = array(
    '#title' => t('Vertical margin'),
    '#type' => 'textfield',
    '#default_value' => $settings['row_setting']['verticalMargin'],
    '#maxlength' => 60,
    '#size' => 60,
  );
  $form['row_setting']['animate'] = array(
    '#type'  => 'checkbox',
    '#title' => t('Animate'),
    '#default_value' => $settings['row_setting']['animate'],
  );
  $form['row_setting']['alwaysShowResizeHandle'] = array(
    '#type'  => 'checkbox',
    '#title' => t('Always show resize handle'),
    '#default_value' => $settings['row_setting']['alwaysShowResizeHandle'],
  );
  $form['row_setting']['auto'] = array(
    '#type'  => 'checkbox',
    '#title' => t('Auto'),
    '#default_value' => $settings['row_setting']['auto'],
  );
  $form['row_setting']['disableDrag'] = array(
    '#type'  => 'checkbox',
    '#title' => t('Disable drag'),
    '#default_value' => $settings['row_setting']['disableDrag'],
  );
  $form['row_setting']['disableResize'] = array(
    '#type'  => 'checkbox',
    '#title' => t('Disable resize'),
    '#default_value' => $settings['row_setting']['disableResize'],
  );
  $form['row_setting']['float'] = array(
    '#type'  => 'checkbox',
    '#title' => t('float'),
    '#default_value' => $settings['row_setting']['float'],
  );

  return $form;
}

/**
 * Implements hook_field_widget_form().
 */
function gridstack_field_field_widget_form(&$form, &$form_state, $field, $instance, $langcode, $items, $delta, $element) {
  switch ($instance['widget']['type']) {
    case 'gridstack_field':
      // Converting options to boolean type for preventing issues
      // with incorrect types.
      $options = _gridstack_field_get_options('bool');
      foreach ($options as $option) {
        $field['settings']['row_setting'][$option] = (bool) $field['settings']['row_setting'][$option];
      }

      // Converting options to int type for preventing issues
      // with incorrect types.
      $options = _gridstack_field_get_options('int');
      foreach ($options as $option) {
        $field['settings']['row_setting'][$option] = intval($field['settings']['row_setting'][$option]);
      }
      // Pass settings into script.
      drupal_add_js(array('gridstack_field' => $field['settings']), array('type' => 'setting'));

      // Add Backbone, Underscore and Gridstack libraries.
      drupal_add_library('gridstack_field', 'gridstack', TRUE);
      drupal_add_library('gridstack_field', 'gridstack_field_lib', TRUE);

      $value = isset($items[$delta]['json']) ? $items[$delta]['json'] : '';
      $element['items'] = array(
        '#markup' => '<div class="gridstack-items"><div class="grid-stack"></div></div>',
      );
      $element['gridstack_group'] = array(
        '#type' => 'fieldset',
        '#collapsible' => FALSE,
        '#collapsed' => FALSE,
        '#title' => check_plain(isset($element['#title']) ? $element['#title'] : ''),
        '#description' => check_plain(isset($element['#description']) ? $element['#description'] : ''),
      );
      $element['gridstack_group']['add_item'] = array(
        '#type' => 'button',
        '#button_type' => 'button',
        '#value' => t('Add item'),
        '#executes_submit_callback' => FALSE,
      );
      $element['gridstack_group']['gridstack_autocomplete'] = array(
        '#type' => 'textfield',
        '#title' => t('Node'),
        '#maxlength' => 60,
        '#autocomplete_path' => 'gridstack_field/' . $field['field_name'] . '/autocomplete',
      );
      $element['json'] = array(
        '#type' => 'textfield',
        '#default_value' => $value,
        '#maxlength' => 2048,
        '#size' => 60,
      );
      break;
  }

  return $element;
}

/**
 * Implements hook_field_formatter_info().
 */
function gridstack_field_field_formatter_info() {
  return array(
    'gridstack_field_formatter' => array(
      'label' => t('Gridstack field'),
      'field types' => array('gridstack_field'),
      'settings'  => array(
        'height' => '0',
        'width' => '12',
        'cellHeight' => '60',
        'minWidth' => '768',
        'rtl' => 'auto',
        'verticalMargin' => '10',
        'animate' => 0,
        'alwaysShowResizeHandle' => 0,
        'auto' => 1,
        'disableDrag' => 0,
        'disableResize' => 0,
        'draggable' => 0,
        'float' => 0,
      ),
    ),
  );
}

/**
 * Implements hook_field_formatter_view().
 */
function gridstack_field_field_formatter_view($entity_type, $entity, $field, $instance, $langcode, $items, $display) {
  $element = array();

  // Processing format types.
  switch ($display['type']) {
    case 'gridstack_field_formatter':
      foreach ($items as $delta => $item) {
        // If we have item then.
        if ($item['json']) {
          $formatted_text = $item['json'];
          // Show text with tags.
          $element[$delta]['#markup'] = $formatted_text;
        }

        // Converting options to boolean type for preventing issues
        // with incorrect types.
        $options = _gridstack_field_get_options('bool');
        foreach ($options as $option) {
          $field['settings']['row_setting'][$option] = (bool) $field['settings']['row_setting'][$option];
        }

        // Converting options to int type for preventing issues
        // with incorrect types.
        $options = _gridstack_field_get_options('int');
        foreach ($options as $option) {
          $field['settings']['row_setting'][$option] = intval($field['settings']['row_setting'][$option]);
        }

        // Pass settings into script.
        drupal_add_js(array('gridstack_field' => $field['settings']), array('type' => 'setting'));

        // Add Backbone, Underscore and Gridstack libraries.
        drupal_add_library('gridstack_field', 'gridstack', TRUE);
        drupal_add_library('gridstack_field', 'gridstack_field_lib', TRUE);
      }
      break;
  }
  return $element;
}

/**
 * Implements hook_field_is_empty().
 */
function gridstack_field_field_is_empty($item, $field) {
  if (empty($item['json'])) {
    return TRUE;
  }
}

/**
 * Return array with keys of options for gridstack plugin separated by type.
 *
 * @param string $type
 *   Determine which type of options should be returned.
 *
 * @return array
 *   An array with options keys.
 */
function _gridstack_field_get_options($type) {
  $options = array();

  switch ($type) {
    case 'bool':
      $options = array(
        'animate',
        'alwaysShowResizeHandle',
        'auto',
        'disableDrag',
        'disableResize',
        'float',
      );
      break;

    case 'int':
      $options = array(
        'height',
        'width',
        'cellHeight',
        'minWidth',
        'rtl',
        'verticalMargin',
      );
      break;

    default:
      break;
  }

  return $options;
}

/**
 * Get enables content types displays.
 */
function _gridstack_field_get_displays($type) {
  $entity_info = entity_get_info('node');
  $view_modes = $entity_info['view modes'];
  $view_mode_settings = field_view_mode_settings('node', $type);
  $displays = array();
  foreach ($view_modes as $view_mode_name => $view_mode_info) {
    if (!empty($view_mode_settings[$view_mode_name]['custom_settings'])) {
      $displays[$view_mode_name] = $view_mode_info['label'];
    }
  }
  return $displays;
}
